package edu.cosy.JESICA.ControllingMaster;

import edu.cosy.JESICA.Interfaces.*;
import edu.cosy.JESICA.Tools.*;
import java.util.*;
import java.rmi.*;



public class Agent implements IAgent {
	
	transient IANNFacade _facade=null;
	public ArrayList	_Way=null;
	public ArrayList	_Bag=null;
	public ArrayList	_WayError=null;
	IWorldCursor 		_cursor=null;
	int 				_nID=0;
	int 				_nMaxWay=100;
	IWorld				_world=null;
	int 				_nStepSize=2;
	double				_dMinError=0.001;
	int 				_nMaxCycles=1000;
	double				_dLearningRate=0.1;
	
	boolean				_bPointAddedToBag=false;

	
	public static final int BAGSIZE=10;
	
	public Agent(int nID,IWorldCursor cursor)
	{
		_nID=nID;
		_Way=new ArrayList();
		_Bag=new ArrayList();
		_WayError=new ArrayList();
		_cursor=cursor;
	};
	
	
	public IAgentState getState() 
	{
		return new AgentState(_Way,_Bag,0,_cursor.getWorld(),_WayError);
	}

	public void setANN(IANNFacade ann) 
	{
		_facade=ann;
	}

	public IANNFacade getANN() 
	{
		return _facade;
	}

	public int getID() 
	{
		return _nID;
	}
	
	
	/**
	 * Binds to a world after network transmission
	 */
	public void bindWorld(IWorld wrld)
	{
		_cursor.bindWorld(wrld);

		Iterator wit=_Way.iterator();
		ArrayList newWay= new ArrayList();
		
		while(wit.hasNext())
		{
			IWorldPoint pt= (IWorldPoint)wit.next();
			pt.bindWorld(wrld);
			newWay.add(pt);
		};
		
		_Way=newWay;
		
		Iterator bit=_Bag.iterator();
		ArrayList newBag= new ArrayList();
		
		while(bit.hasNext())
		{		
			IWorldPoint pt= (IWorldPoint)bit.next();
			pt.bindWorld(wrld);
			newBag.add(pt);
		}
		
		_Bag=newBag;
		
	}

	public void performTraining(IAgent.IAgentTrainingInterrupt interrupter)
	{
		
		// Fill agents bag:
		_Bag.clear();
		_bPointAddedToBag=true;
		
		for(int i=0; i<BAGSIZE;i++) 
		{
			_Bag.add(_cursor.getPoint());
			_Way.add(_cursor.getPoint());
			_WayError.add(new Double(0));
			doMove();
		};
		

		try
		{		
			// perform training on bag
			
			newNet(_Bag);
			_facade.performTraining(interrupter);
		}
		catch(RemoteException e)
		{
			System.out.println(e.toString());
		};

		
		
		// move and learn further
		for( int j=0 ; !interrupter.isInterrupted() && j < _nMaxWay; j++)
		{
			
			doMove();
			IWorldPoint point=_cursor.getPoint();
			_Way.add(point);
			
			
			IWorldPoint[] valid=createPointField(_Way);
			
			try
			{
				// Train on old bag and validate with new way
				newNet(_Bag);
				_facade.performTraining(interrupter);
	
				double oldErr= _facade.getMSE(valid);
				double wayErr= Double.MAX_VALUE;
				int oldIndex= -1;

				IWorldPoint ptBackup=null;
				
				
				// Try different bag configurations
				for(int i=0;i<BAGSIZE;i++)
				{
					ptBackup= (IWorldPoint)_Bag.get(i);
					
					_Bag.remove(i);
					_Bag.add(i,point);
					
					newNet(_Bag);
					_facade.performTraining(interrupter);
					
					double err= _facade.getMSE(valid);
									
					if(err<oldErr) oldIndex=i;
					
					if(err<wayErr) wayErr= err;
					
					_Bag.remove(i);
					_Bag.add(i,ptBackup);
	
					if(interrupter.isInterrupted()) return;
				};
					
					
//				System.out.println("Agent "+ getID() +" training " + j +" finished, swapping Bagindex to " + oldIndex +"; oldErr: " + oldErr + ", wayErr: " +wayErr);
				System.out.println("Agent "+ getID() +" training " + j +" finished, swapping Bagindex to " + oldIndex );
				
				if(oldIndex >=0)
				{
					_Bag.remove(oldIndex);
					_Bag.add(oldIndex,point);
					_bPointAddedToBag=true;
				}
				else
				{
					_bPointAddedToBag=false;
				}
				
				_WayError.add(new Double(wayErr));
			}
			catch(RemoteException e)
			{
				System.out.println(e.toString());
			};
		
		}		
		
		System.out.println("Finished Agent " + getID());
		
		return;
	}
	
	
	private void newNet(ArrayList bag) throws RemoteException
	{
		_facade.newNet();
		_facade.setMinimumError(_dMinError);
		_facade.setMaxTrainingCycles(_nMaxCycles);
		_facade.setLearningRate(_dLearningRate);
		
		_facade.setPatternSet(createPointField(bag));
	}
		
		
	
	
	public IWorldPoint[] createPointField(ArrayList ptCol)
	{
		IWorldPoint[] points= new IWorldPoint[ptCol.size()];
		
		for(int i=0;i<points.length;i++)
		{
			points[i]=(IWorldPoint)ptCol.get(i);
		};
		
		return points;
	}
		
	
	public void doMove()
	{
		IWorldPoint pt;
		do
		{			
			MoveStep();
			pt=_cursor.getPoint();
		}
		while(!pt.hasData() || isInWay(pt,_Bag));
	}
		
	
	
	protected void MoveStep()	
	{
		int direction=Randomizer.random(IWorldCursor.DIRECTION_NUMBERS.EAST,IWorldCursor.DIRECTION_NUMBERS.SE);

		MoveDirection(direction);
		
	}		
	
		
	protected void MoveDirection(int direction)
	{
		switch(direction)
		{
			case IWorldCursor.DIRECTION_NUMBERS.WEST: 
					for(int i=0; i<_nStepSize;i++) _cursor.goWest();
					break;
					
			case IWorldCursor.DIRECTION_NUMBERS.NW: 
				 for(int i=0; i<_nStepSize;i++) _cursor.goNW(); 
					break;
					
			case IWorldCursor.DIRECTION_NUMBERS.NORTH: 
				 for(int i=0; i<_nStepSize;i++) _cursor.goNorth(); 
					break;
					
			case IWorldCursor.DIRECTION_NUMBERS.NE: 
				 for(int i=0; i<_nStepSize;i++) _cursor.goNE(); 
					break;
					
			case IWorldCursor.DIRECTION_NUMBERS.EAST: 
				 for(int i=0; i<_nStepSize;i++) _cursor.goEast(); 
					break;
					
			case IWorldCursor.DIRECTION_NUMBERS.SE: 
				 for(int i=0; i<_nStepSize;i++) _cursor.goSE(); 
					break;
					
			case IWorldCursor.DIRECTION_NUMBERS.SOUTH: 
				 for(int i=0; i<_nStepSize;i++) _cursor.goSouth(); 
					break;
					
			case IWorldCursor.DIRECTION_NUMBERS.SW: 
				 for(int i=0; i<_nStepSize;i++)  _cursor.goSW(); 
					break;
					
			default: for(int i=0; i<_nStepSize;i++) _cursor.goNorth();
		};
	}		


	public boolean isInWay(IWorldPoint pt, ArrayList way)
	{
		Iterator it= way.iterator();
		
		while(it.hasNext())
		{
			IWorldPoint oldpt=(IWorldPoint)it.next();
			
			if(oldpt.getX()==pt.getX() && oldpt.getY()==pt.getY()) return true;
		};
		
		return false;
	}
	
	
	/**	
	 * Queries the neural net for classifikation of the given point
	 */
	public float queryResult(IWorldPoint pt)
	{
		try
		{
			return (float)_facade.getClassfication(pt);
		}
		catch(RemoteException e)
		{
			System.out.println(e.toString());
			return Float.NaN;
		};
	}



	public void setStepsize(int nStep)
	{
		_nStepSize=nStep;
	}
	
	public void setMinimumError(double dMinError)
	{
		_dMinError=dMinError;
	}
	
	public void setMaxTrainingCycles(int nMaxCycles)
	{
		_nMaxCycles=nMaxCycles;
	}
	
	
	public void setMaxWaylength(int nMaxWay)
	{
		_nMaxWay=nMaxWay;
	}

	public void setLearningRate(double dLearn)
	{
		_dLearningRate=dLearn;
	}



	class AgentState implements IAgentState
	{
		public IWorldPoint[]	_Way	={};
		public IWorldPoint[]	_Bag	={};
		public float			_Error	=0;
		public IWorldRect		_BoundingBox =null;
		public double[]			_ErrWay ={};
		
		AgentState(ArrayList way, ArrayList bag, float mse, IWorld world, ArrayList errway)
		{
			_Error=mse;
			setWay(way,world);
			setBag(bag,world);
			setErrorWay(errway);
		}
		
			
		void setWay(ArrayList way, IWorld world)
		{
			_Way= new IWorldPoint[way.size()];

			for(int i=0;i<_Way.length;i++)
			{
				_Way[i]=(IWorldPoint)way.get(i);
				//_Way[i].bindWorld(world);
			}
			
			
			if(_Way.length<1) 
			{
				_BoundingBox=null;
			}
			else
			{
				//IWorldPoint pt=_Way[0];
				//IWorld wrld= pt._world;
				
				if(world==null)
				{
					System.out.println("Error: No World in AgentState::setWay!");
					return;
				};
				
				_BoundingBox = world.getBoundingRect();
				_BoundingBox.setRect(_Way[0],_Way[0]);
				
				for(int i=0;i<_Way.length;i++)
				{
					_BoundingBox.enlargeBounding(_Way[i]);
				};
			};
			
		}
			
		void setBag(ArrayList bag, IWorld world)
		{
			_Bag= new IWorldPoint[bag.size()];

			for(int i=0;i<_Bag.length;i++)
			{
				_Bag[i]=(IWorldPoint)bag.get(i);
			}
		}

		void setErrorWay(ArrayList errway)
		{
			_ErrWay= new double[errway.size()];

			for(int i=0;i<_ErrWay.length;i++)
			{
				_ErrWay[i]=((Double)errway.get(i)).doubleValue();
			}
		}
		
		public IWorldRect getBoundingBox()
		{
			return _BoundingBox;
		}
		
		public float getError()
		{
			return _Error;
		}


		public IWorldPoint[] getBag()
		{
			return _Bag;
		}


		public IWorldPoint [] getWay()
		{
			return _Way;
		}
		
		
		public double[]	getWayError()
		{
			return _ErrWay;
		}

		public void bindWorld(IWorld wrld)
		{
			for(int i=0;i<_Bag.length;i++)
			{
				_Bag[i].bindWorld(wrld);
			}
			
			for(int i=0;i<_Way.length;i++)
			{
				_Way[i].bindWorld(wrld);
			}
			
			if(_BoundingBox!=null) _BoundingBox.bindWorld(wrld);
			
		}
		
	}


}
