package edu.cosy.JESICA.Interfaces;

/** 
	* Interface for mobile position marker in abstract world
	* @author taschaue
	*/
public interface IWorldCursor extends java.io.Serializable
{
	/**
	 * The cursors binded world
	 */
	//protected IWorld	_world	=null;

	/**
		* Go one step north.
		* @return false if the cursor is already at the world's frontier
		*/
	public boolean goNorth();
	
	/**
		* Go one step south.
		* @return false if the cursor is already at the world's frontier
		*/
	public boolean goSouth();

	/**
		* Go one step west.
		* @return false if the cursor is already at the world's frontier
		*/
	public boolean goWest();

	/**
		* Go one step East.
		* @return false if the cursor is already at the world's frontier
		*/
	public boolean goEast();

	/**
		* Go one step north west.
		* @return false if the cursor is already at the world's frontier
		*/
	public boolean goNW();

	/**
		* Go one step north east.
		* @return false if the cursor is already at the world's frontier
		*/
	public boolean goNE();

	/**
		* Go one step south west.
		* @return false if the cursor is already at the world's frontier
		*/
	public boolean goSW();

	/**
		* Go one step south east.
		* @return false if the cursor is already at the world's frontier
		*/
	public boolean goSE();


	/**
		* Move cursor to point
		* @return false if the cursor would exceed the world's frontier
		*/
	public boolean gotoPoint(IWorldPoint pt);

	/**
		* Move cursor to point
		* @return false if the cursor would exceed the world's frontier
		*/
	public boolean gotoPoint(int x, int y);

	/**
		* Move cursor to point
		* Note: Does not check if the point is in the world's frontier
		*/
	public void gotoPointUnchecked(int x, int y);

	
	/**
		* Return the cursors current location
		*/
	public IWorldPoint getPoint();
	
	
	/**
	 * Binds a world after network transmission
	 */
	public void bindWorld(IWorld wrld);

	/**
	 * Returns the currently bounden world
	 */	
	public IWorld getWorld();
	
	
	public interface DIRECTION_NUMBERS
	{
		
		public static final int EAST	=0;
		public static final int NE		=1;
		public static final int NORTH	=2;
		public static final int NW		=3;
		public static final int WEST	=4;
		public static final int SW		=5;
		public static final int SOUTH	=6;
		public static final int SE		=7;
	}

}

