package edu.cosy.JESICA.Test;

import edu.cosy.JESICA.ControllingMaster.*;
import edu.cosy.JESICA.Interfaces.*;
import edu.cosy.JESICA.Tools.*;
import junit.framework.*;


public class TestTools extends TestCase{
	
	
	/**
	 * Default constructor for JUnit test case
	 */
	public TestTools(String arg1)
	{
		super(arg1);
	}
	
	
	/**
	 * Main method running the test case
	 */
	public static void main(String[] args) 
	{
		junit.textui.TestRunner.run(suite());
	}	
	
	/**
	 * Helper function for getting the class of the test case
	 */
	public static Test suite()
	{
		return new TestSuite(TestTools.class);
	}

	private static final int _worldSize=20;

	
	
	/**
	 * Helper function creating a discrete world
	 */
	public DiscreteWorld createWorld()
	{
		DataPoints[] pts =  new DataPoints[_worldSize*_worldSize];
		DiscreteWorld world= new DiscreteWorld();
		
		for(int i=0;i<_worldSize;i++)
		{			
			for(int k=0;k<_worldSize;k++)
			{
			
			DataPoints p=new DataPoints();
			
			assertNotNull("i="+i+", k=" +k+", pts.length="+pts.length,p);
			
			p.x=k;
			p.y=i;
			
			int pos = i*_worldSize+k;
			
			p.d=pos;
			
			pts[pos]=p;
			
			
			};
			
		}
		
		world.setData(pts);
		return world;
	}

	/**
	 * Helper function comparing two world points
	 */
	public void assertPoint(String str, IWorldPoint pt1, IWorldPoint pt2)
	{
		assertEquals(str+" [x]",pt1.getX(),pt2.getX());
		assertEquals(str+" [y]",pt1.getY(),pt2.getY());
	}



	/**
	 * Test method: test Centroid Calculation
	 */	
	public void testCentroidCalculation() throws Exception
	{
    	DiscreteWorld world=createWorld();
    	
    	
    	DiscreteWorldPoint pt1= new     	DiscreteWorldPoint(world,10,0);
    	DiscreteWorldPoint pt2= new     	DiscreteWorldPoint(world,10,10);
    	DiscreteWorldPoint pt3= new     	DiscreteWorldPoint(world,0,10);
    	DiscreteWorldPoint pt4= new     	DiscreteWorldPoint(world,5,10);
    	DiscreteWorldPoint pt5= new     	DiscreteWorldPoint(world,15,10);
    	DiscreteWorldPoint pt6= new     	DiscreteWorldPoint(world,5,10);
    	DiscreteWorldPoint pt7= new     	DiscreteWorldPoint(world,5,0);
    	DiscreteWorldPoint pt8= new     	DiscreteWorldPoint(world,15,0);
    	DiscreteWorldPoint pt9= new     	DiscreteWorldPoint(world,15,-10);
    	
    	
    	WorldVector v1= new WorldVector(pt1,pt2);
    	WorldVector v2= new WorldVector(pt3,pt4);
    	WorldVector v3 = PointTools.addVector(v1,v2);
    	
    	assertPoint("StartPoint <1>",pt1,v3.getStartPoint());
    	assertPoint("EndPoint <1>",pt5,v3.getEndPoint());
    	
    	System.out.println("V1 <1>: angle= " + v1.getAngle() + ", length="+v1.getLength());
    	System.out.println("V2 <1>: angle= " + v2.getAngle() + ", length="+v2.getLength());
    	System.out.println("V3 <1>: angle= " + v3.getAngle() + ", length="+v3.getLength());

		v1= new WorldVector(pt1,pt2);
		v2= new WorldVector(pt4,pt3);
    	v3 = PointTools.addVector(v1,v2);

    	assertPoint("StartPoint <2>",pt1,v3.getStartPoint());
    	assertPoint("EndPoint <2>",pt6,v3.getEndPoint());

    	System.out.println("V1 <2>: angle= " + v1.getAngle() + ", length="+v1.getLength());
    	System.out.println("V2 <2>: angle= " + v2.getAngle() + ", length="+v2.getLength());
    	System.out.println("V3 <2>: angle= " + v3.getAngle() + ", length="+v3.getLength());
		    	

		v1= new WorldVector(pt2,pt1);
		v2= new WorldVector(pt4,pt3);
    	v3 = PointTools.addVector(v1,v2);

    	assertPoint("StartPoint <3>",pt2,v3.getStartPoint());
    	assertPoint("EndPoint <3>",pt7,v3.getEndPoint());

    	System.out.println("V1 <3>: angle= " + v1.getAngle() + ", length="+v1.getLength());
    	System.out.println("V2 <3>: angle= " + v2.getAngle() + ", length="+v2.getLength());
    	System.out.println("V3 <3>: angle= " + v3.getAngle() + ", length="+v3.getLength());
		

		v1= new WorldVector(pt2,pt1);
		v2= new WorldVector(pt3,pt4);
    	v3 = PointTools.addVector(v1,v2);

    	assertPoint("StartPoint <4>",pt2,v3.getStartPoint());
    	assertPoint("EndPoint <4>",pt8,v3.getEndPoint());

    	System.out.println("V1 <4>: angle= " + v1.getAngle() + ", length="+v1.getLength());
    	System.out.println("V2 <4>: angle= " + v2.getAngle() + ", length="+v2.getLength());
    	System.out.println("V3 <4>: angle= " + v3.getAngle() + ", length="+v3.getLength());
		
		
		v1= new WorldVector(pt2,pt1);
		v2= new WorldVector(pt3,pt4);
    	v3 = PointTools.addVector(v1,v2);
    	v3 = PointTools.addVector(v3,v1);

    	assertPoint("StartPoint <5>",pt2,v3.getStartPoint());
    	assertPoint("EndPoint <5>",pt9,v3.getEndPoint());

    	System.out.println("V1 <5>: angle= " + v1.getAngle() + ", length="+v1.getLength());
    	System.out.println("V2 <5>: angle= " + v2.getAngle() + ", length="+v2.getLength());
    	System.out.println("V3 <5>: angle= " + v3.getAngle() + ", length="+v3.getLength());
		
		
	}


}