package edu.cosy.JESICA.WorkingSlave;

import edu.cosy.JESICA.Interfaces.*;
import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;
import java.util.*;


public class SlaveController extends UnicastRemoteObject
implements IWorkingSlaveController 
{
	// list of all agents in the Slave
  	public  ArrayList	_AgentList = null;
	// list of all Agentthreads
  	public  ArrayList	_AgentThreads = null;
  	// world for the slaves
  	public  IWorld		_World = null;

  	
	// Construction
	public SlaveController() throws RemoteException
	{   super();	// call superclass constructor to export this object
		_AgentList = new ArrayList();
		_AgentThreads = new ArrayList();		
	}
	
	// Main method for registering object
	public static void main(String[] args) 
	{
		try
		{
			System.out.println("Welcome to JESICA Working Slave!");
			SlaveController slave=new SlaveController();

			System.out.println("Slave instantiated: " +slave);

			Naming.rebind("/JESICA_WorkingSlave",slave);
			System.out.println("Slave bound to registry.");
			
		}
		catch(Exception e)
		{
			System.out.println("ERROR: Exception in slave main: "+e);
		};
	}
	
	
	IControllingMasterCallbacks _cbMaster=null;
	 
	
	public void setCallback(IControllingMasterCallbacks cbMaster) throws RemoteException
	{
		_cbMaster=cbMaster;
	}
	
	
	/** Adds an agent to working slave
	 */
	public void addAgent(IAgent agent) throws RemoteException {
		// TODO: Add your code here
		System.out.println("Remote call: addAgent("+agent+")");
		
		// bind to local world
		agent.bindWorld(_World);
		
		// add agent to list		
		_AgentList.add(agent);
		
		// create thread for agent
		_AgentThreads.add( new AgentThread( agent, _cbMaster ) );
			
	}
	
	/** Starts training of an agent
	 */
	public void trainAgent(int nAgentID) throws RemoteException {
		// TODO: Add your code here
		System.out.println("Remote call: trainAgent("+nAgentID+")");

		// get agent
		IAgent agent = getAgent(nAgentID);
		if( agent == null ) {
			System.out.println("Can't find agent to start training");
			return ;
		}
		
		// get thread
		AgentThread agentT = null;
		
		Iterator it = _AgentThreads.iterator();
		// search for thread
		while( it.hasNext() ) {
			agentT = (AgentThread)it.next();
			if( agentT._Agent == agent ) break;
			else agentT = null;
		}
		
		// creates new agentthread if none was found				
		if( agentT == null ) {
			_AgentThreads.add( new AgentThread( agent, _cbMaster ) );
		}

		// start training // thread
		agentT.trainAgent();
		
	}

	/** removes an agent from working slave
	 */
	public void removeAgent(int nAgentID) throws RemoteException {
		// TODO: Add your code here
		System.out.println("Remote call: removeAgent("+nAgentID+")");
		
		// get agent
		IAgent agent = getAgent(nAgentID);
		if( agent == null ) return ;
		// remove agent thread
		Iterator it = _AgentThreads.iterator();
		AgentThread agentT = null;
		// search for thread
		while( it.hasNext() ) {
			agentT = (AgentThread)it.next();
			if( agentT._Agent == agent ) {
				// agent thread found
				// interrupt agent if still active
				if( agentT.isAlive() ) agentT.interrupt();
				// remove agent
				it.remove();
			}
		}
	}
	
	/** Get state of a specified agent
	 */
	public IAgentState getAgentState(int nAgentID) throws RemoteException {
		// TODO: Add your code here
		System.out.println("Remote call: getAgentState("+nAgentID+")");
		
		Iterator it = _AgentList.iterator();
		IAgent agent;
		// search for agent
		while( it.hasNext() ) {
			
			agent = (IAgent)it.next();
			
			if( agent.getID() == nAgentID) {
				// return agent status
				return agent.getState();
			}
		}
		// agent not found
		return null;
	}
	
	/** get an agent 
	 */
	public IAgent getAgent(int nAgentID) throws RemoteException {
		// TODO: Add your code here
		System.out.println("Remote call: getAgent("+nAgentID+")");
		
		Iterator it = _AgentList.iterator();
		IAgent agent;
		// search for agent
		while( it.hasNext() ) {
			
			agent = (IAgent)it.next();
			
			if( agent.getID() == nAgentID) {
				// return agent status
				return agent;
			}
		}
		// agent not found
		return null;
	}
	
	/** Set world for slaves agents 
	 */
	public void setWorld(IWorld world) throws RemoteException {
		// TODO: Add your code here
		System.out.println("Remote call: setWorld("+world+")");
		
		_World = world;
		_World.DebugWorld();
	}
	
	 /**
	  * Get the ANN-Facade of the given agent
	  */
	 public IANNFacade getFacade(int nAgentID) throws RemoteException {
	 	
		// get agent
		IAgent agent = getAgent(nAgentID);
		if( agent != null)	return agent.getANN();	 		 	
		
		System.out.println("Can't return facade of a non existing agent !");
		return null;
	 }
	
}
