package edu.cosy.netgen.ann;

import java.beans.*;
import java.awt.*;
import java.awt.event.*;
import java.net.URL;
import java.util.*;

import javax.swing.*;

import edu.cosy.netgen.ann.ui.*;

/**
 * Wizard for customizing the ANN bean.
 */
public class ANNBeanWizard extends JPanel implements Customizer {
    
    // current wizard page
    private int currentPage = 0;
    
    // panel for wizard pages
    private JPanel pages;
    
    // label for error messages
    private JLabel errorLabel;
    
    // wizard pages    
    private WizardPage[] wizardPages;
    
    // control buttons
    private JButton[] controls;
    
    // working values for the wizard
    private Hashtable beanValues = null;
    
    // actual bean
    private ANN bean;
    
    /**
     * Create a new wizard for customizing the bean.
     */
    public ANNBeanWizard() {
        super();
        setLayout( new BorderLayout() );
        
        // load logo image
        URL url = getClass().getResource( "/edu/cosy/netgen/ann/ui/bean.jpg" );
        if ( url != null )
            add( new JLabel( new ImageIcon( url ) ), BorderLayout.WEST );
        
        // create wizard pages
        wizardPages = new WizardPage[]{ new StructureWizardPage( "Network Structure" ), 
                                        new ActivationWizardPage( "Activation Functions" ),
                                        new TrainingWizardPage( "Training Function" ), 
                                        new TrainingParamsWizardPage( "Training Paramters" ) };        
                                        
        // create all other UI elements                                       
        pages = createPagesPanel();
        errorLabel = new JLabel();
        errorLabel.setOpaque( true );
        errorLabel.setBackground( Color.red);
        errorLabel.setForeground( Color.white );
        JPanel tmp = new JPanel();
        tmp.setLayout( new BorderLayout() );
        tmp.add( pages, BorderLayout.CENTER );
        tmp.add( errorLabel, BorderLayout.SOUTH );
        add( tmp, BorderLayout.CENTER );
        add( createControlsPanel(), BorderLayout.SOUTH );
        
        // enable/disable controls
        enableControls();
    }

    private JPanel createPagesPanel() {
        JPanel retVal = new JPanel();
        retVal.setLayout( new CardLayout() );
        for ( int i = 0; i < wizardPages.length; i++ )
            retVal.add( "page_" + i, wizardPages[ i ] );
        return retVal;
    }
        
    private JPanel createControlsPanel() {
        JPanel retVal = new JPanel();
        controls = new JButton[ 3 ];
        URL url = getClass().getResource( "/edu/cosy/netgen/ann/ui/left.gif" );
        controls[ 0 ] = new JButton( "Previos", new ImageIcon( url ) );
        controls[ 0 ].addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                prevButtonAction( e );
            }
        } );
        retVal.add( controls[ 0 ] );
        url = getClass().getResource( "/edu/cosy/netgen/ann/ui/right.gif" );
        controls[ 1 ] = new JButton( "Next", new ImageIcon( url ) );
        controls[ 1 ].setHorizontalTextPosition( JButton.LEADING );
        controls[ 1 ].addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                nextButtonAction( e );
            }
        } );
        retVal.add( controls[ 1 ] );
        url = getClass().getResource( "/edu/cosy/netgen/ann/ui/save.gif" );
        controls[ 2 ] = new JButton( "Apply", new ImageIcon( url ) );
        controls[ 2 ].addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                applyButtonAction( e );
            }
        } );
        retVal.add( controls[ 2 ] );
        return retVal;
    }
    
    public void setObject( Object obj ) {
        bean = ( ANN ) obj;
        if ( beanValues == null )
            beanValues = new Hashtable();
        beanValues.put( "description", bean.getDescription() );
        beanValues.put( "netType", new Integer( bean.getNetType() ) );
        beanValues.put( "conStyle", new Integer( bean.getConnectionStyle() ) );
        beanValues.put( "in", new Integer( bean.getIn() ) );
        beanValues.put( "hidden", new Integer( bean.getHidden() ) );
        beanValues.put( "out", new Integer( bean.getOut() ) );
        beanValues.put( "actFunctions", bean.getActivationFunctionsByNames() );
        beanValues.put( "fitFunction", bean.getFitnessFunctionByName() );
        if ( bean.getRegFunction() == null )
            beanValues.put( "regFunction", "None" );
        else            
            beanValues.put( "regFunction", bean.getRegFunctionByName() );
        beanValues.put( "trainFunction", bean.getTrainingFunctionByName() );
        for ( int i = 0; i < wizardPages.length; i++ ) {
            wizardPages[ i ].setBean( bean );
            wizardPages[ i ].setValues( beanValues );
        }
        wizardPages[ 0 ].initialize();
   }      
    
    private void prevButtonAction( ActionEvent e ) {
        if ( currentPage > 0 ) {            
            CardLayout layout = ( CardLayout ) pages.getLayout();
            layout.previous( pages );
            currentPage--;
            enableControls();
        }
    }

    private void nextButtonAction( ActionEvent e ) {
        if ( wizardPages.length > currentPage + 1 ) {
            try {
                wizardPages[ currentPage ].check();
            } catch ( ANNBeanWizardException bwe ) {
                errorLabel.setText( "Error: " + bwe.getMessage() );
                return;
            }
            wizardPages[ currentPage + 1 ].initialize();
            CardLayout layout = ( CardLayout ) pages.getLayout();
            layout.next( pages );
            currentPage++;
            enableControls();
        }   
    }

    private void applyButtonAction( ActionEvent e ) {
        errorLabel.setText( null );
        try {
            wizardPages[ currentPage ].check();
        } catch ( ANNBeanWizardException bwe ) {
            errorLabel.setText( "Error: " + bwe.getMessage() );
            return;
        }
        try {
            bean.setDescription( ( String ) beanValues.get( "description" ) ); 
            bean.setNetType( ( ( Integer ) beanValues.get( "netType" ) ).intValue() ); 
            bean.setConnectionStyle( ( ( Integer ) beanValues.get( "conStyle" ) ).intValue() );  
            bean.setIn( ( ( Integer ) beanValues.get( "in" ) ).intValue() );
            bean.setHidden( ( ( Integer ) beanValues.get( "hidden" ) ).intValue() );
            bean.setOut( ( ( Integer ) beanValues.get( "out" ) ).intValue() );
            bean.setActivationFunctionsByNames( ( String[] ) beanValues.get( "actFunctions" ) );
            bean.setFitnessFunctionByName( ( String ) beanValues.get( "fitFunction" ) );
            String tmp = ( String ) beanValues.get( "regFunction" );
            if ( tmp != null && tmp.equals( "None" ) )
                tmp = null;
            bean.setRegFunctionByName( tmp );
            bean.setTrainingFunctionByName( ( String ) beanValues.get( "trainFunction" ) );
            TrainingParam[] params = ( TrainingParam[] ) beanValues.get( "trainParams" );
            if ( params != null )
                bean.setTrainingFunctionParams( params );
        } catch ( Exception ex ) {
            System.err.println( "XXX:" + ex );
        }
    }
        
    private void enableControls() {
        errorLabel.setText( null );
        controls[ 0 ].setEnabled( currentPage > 0  ? true : false );
        controls[ 1 ].setEnabled( currentPage < wizardPages.length - 1  ? true : false );
        controls[ 2 ].setEnabled( currentPage == wizardPages.length - 1  ? true : false );
    }
}
