package edu.cosy.netgen.ann;

public class FitnessFunctionMSE extends FitnessFunction {

    /**
     * Create a new mean square error fitness function
     */
    public FitnessFunctionMSE() {
        super();
    }

    /**
     * Creates a new fitness function instance
     * @param patterns patterns
     * @param ffn net structure
     */
    public FitnessFunctionMSE( PatternSet validSet, FeedForwardNet ffn ) {
        super();
        init( validSet, ffn );
    }

    /**
     * Calculates fitness of the net
     * @return fitness of the net
     */
    public double getFitness() {
        
        //errors are summed up, all patterns, all neurons
        double sum = 0.0;
        
        //for all patterns
        for ( int j = 0; j < validationSet.getSize(); j++ ) {
            
            //forward step
            fpVali.initForwardCycle( j );
            fpVali.forwardCycle();
                
            //for all output neurons
            for ( int i = 0; i < ffn.getOut(); i++ ) {
                sum += sqr( validationSet.getOutPatternArray()[ j ][ i ] 
                            - fpVali.outputvek[ i + 1 + ffn.getIn() + ffn.getHidden() ] );
            }
        }
        
        //0: best fitness, 1:worst fitness
        if ( validationSet.getSize() == 0 )
            return 1.0;
        else
            return 1 - ( 1.0 / ( 1.0 + sum ) );
    }
}
