package edu.cosy.netgen.ann;
 
public class PatternSet  implements java.io.Serializable {
    private boolean[] patternMarkers;
    private String description = null;
    private Pattern[] inPatterns;
    private Pattern[] outPatterns;
    
    public PatternSet() {
        this( new Pattern[]{}, new Pattern[]{} );
    }
    
    public PatternSet( Pattern[] in, Pattern[] out ) {
        super();
        this.inPatterns = in;
        this.outPatterns = out;
        patternMarkers = new boolean[ getSize() ];
        for ( int i = 0; i < getSize(); i++ )
            patternMarkers[ i ] = false;
    }
    
    public PatternSet( double[][] in, double[][] out ) {
        super();
        if ( in != null )
            setInPatternArray( in );
        if ( out != null )
            setOutPatternArray( out );
        patternMarkers = new boolean[ getSize() ];
        for ( int i = 0; i < getSize(); i++ )
            patternMarkers[ i ] = false;
    }
    
    public PatternSet( PatternSet value ) {
        super();        
        this.patternMarkers = new boolean[ value.patternMarkers.length ];
        System.arraycopy( value.patternMarkers, 0, patternMarkers, 0, patternMarkers.length );
        this.description = new String( description );
        this.inPatterns = new Pattern[ value.getInPatterns().length ];
        System.arraycopy( value.getInPatterns(), 0, inPatterns, 0, inPatterns.length );
        this.outPatterns = new Pattern[ value.getOutPatterns().length ];
        System.arraycopy( value.getOutPatterns(), 0, outPatterns, 0, inPatterns.length );
    }
    
    public void setDescription( String desc ) {
        description = desc;
    }
    
    public String getDescription() {
        return description;
    }
    
    public void setInPatterns( Pattern[] in ) {
        inPatterns = in;
    }
    
    public void setOutPatterns( Pattern[] out ) {
        outPatterns = out;
    }
    
    public Pattern[] getInPatterns() {
        return inPatterns;
    }
    
    public Pattern[] getOutPatterns() {
        return outPatterns;
    }
    
    public void setInPatternArray( double[][] val ) {
        inPatterns = new Pattern[ val.length ];
        for ( int i = 0; i < inPatterns.length; i++ ) {
            inPatterns[ i ] = new Pattern( val[ i ] );
        }
    }

    public void setOutPatternArray( double[][] val ) {
        outPatterns = new Pattern[ val.length ];
        for ( int i = 0; i < outPatterns.length; i++ ) {
            outPatterns[ i ] = new Pattern( val[ i ] );
        }
    }
    
    public double[][] getInPatternArray() {
        double[][] retVal = null;
        if ( inPatterns != null && inPatterns.length > 0 ) {
            retVal = new double[ inPatterns.length ][ inPatterns[ 0 ].getSize() ];
            for ( int i = 0; i < inPatterns.length; i++ ) {
                retVal[ i ] = inPatterns[ i ].getValues();
            }
        }
        return retVal;
    }

    public double[][] getOutPatternArray() {
        double[][] retVal = null;
        if ( outPatterns != null && outPatterns.length > 0 ) {
            retVal = new double[ outPatterns.length ][ outPatterns[ 0 ].getSize() ];
            for ( int i = 0; i < outPatterns.length; i++ ) {
                retVal[ i ] = outPatterns[ i ].getValues();
            }
        }
        return retVal;
    }
    
    public int getSize() {
        int retVal = -1;
        if ( inPatterns != null && outPatterns != null && inPatterns.length == outPatterns.length )
            retVal = inPatterns.length; 
        return retVal;
    }

    public boolean[] getMarkers() {
        return patternMarkers;
    }

    public boolean isMarker( int patPos ) {
        if ( patternMarkers != null && patPos < patternMarkers.length )
            return patternMarkers[ patPos ];
        else
            return false;
    }

    public void setMarkers( boolean[] patternMarkers ) {    
        this.patternMarkers = patternMarkers;
    }

    public int getUsedPatternNr() {
        int retVal = 0;
        if ( patternMarkers != null ) {
            for ( int i = 0; i < patternMarkers.length; i++ ) {
                if( !isMarker( i ) )
                    retVal++;
            }
        }
        return retVal;
    }
}
