package edu.cosy.netgen.client.worker;
 
import java.util.Vector;

import edu.cosy.netgen.ann.PatternSet;
import edu.cosy.netgen.ann.ANN;
import edu.cosy.netgen.ga.Individual;
import edu.cosy.netgen.ga.ANNGenotype;
import edu.cosy.netgen.ga.ANNGenotypeTPSMarkers;
import edu.cosy.netgen.server.service.WorkingClientService;
import edu.cosy.netgen.server.TaskCreationInfo;
import edu.cosy.netgen.client.worker.event.FitnessWorkerEvent;
import edu.cosy.netgen.client.worker.event.FitnessWorkerListener;

public class FitnessWorkerThread extends Thread {
    public static final int WAITING = 100;
    public static final int WORKING = 101;
    
    private Vector listeners;
    private WorkingClientService container;
    private Object currentWorkingObject;
    private TrainingProgressAdapter progressListener;
    
    private boolean isSuspended = true;
    
    class TrainingProgressAdapter implements edu.cosy.netgen.ann.ProgressListener {
        private FitnessWorkerThread parent;
        
        public TrainingProgressAdapter( FitnessWorkerThread p ) {
            parent = p;
        }
        
        public void progressMade( edu.cosy.netgen.ann.ProgressEvent e ) {
            notifyProgressMadeEvent( new FitnessWorkerEvent( parent, e.getPercent() ) );                
        }
    }    
    
    public FitnessWorkerThread( WorkingClientService c, Object o ) {
        super();
        container = c;
        currentWorkingObject = o;
        progressListener = new TrainingProgressAdapter( this );
    }
        
    public void run() {
        if ( container != null ) {
            try {
                Individual ind;
                double val;
                while ( true ) {
                    synchronized ( this ) {
                        while ( isSuspended ) {
                            notifyStatusChangedEvent( new FitnessWorkerEvent( this, WAITING, 0 ) );
                            wait();
                        }
                    }
                    val = 0.0;
                    ind = container.getNextIndividual();
                    if ( ind != null && currentWorkingObject != null ) {
                        int index = ind.getIndex();
                        notifyStatusChangedEvent( new FitnessWorkerEvent( this, WORKING, index, container.getCurrentTaskType(), 0 ) );
                        switch ( container.getCurrentTaskType() ) {
                            case TaskCreationInfo.EVOLVE_ANN: {
                                PatternSet[] pats = ( PatternSet[] ) currentWorkingObject;
                                ANNGenotype agt = ( ANNGenotype ) ind.getGenotype();  
                                agt.setPatternSets( pats[ 0 ], pats[ 1 ] );
                                if ( progressListener == null )
                                    val = agt.getRawFitness();
                                else
                                    val = agt.getRawFitness( progressListener );
                                break;
                            }
                            case TaskCreationInfo.EVOLVE_PATTERN: {
                                Vector tmp = ( Vector ) currentWorkingObject;
                                ANN bean = ( ANN ) tmp.get( 0 );
                                PatternSet[] pats = ( PatternSet[] ) tmp.get( 1 );
                                bean.setTrainingPatternSet( pats[ 0 ] );
                                bean.setValidationPatternSet( pats[ 1 ] );
                                ANNGenotypeTPSMarkers agtm = ( ANNGenotypeTPSMarkers ) ind.getGenotype();
                                agtm.setBean( bean );
                                if ( progressListener == null )
                                    val = agtm.getRawFitness();
                                else
                                    val = agtm.getRawFitness( progressListener );
                                break;
                            }
                        }
                        System.out.println( index + ": " + val );
                        container.setFitness( index, val );
                    }
                }
            }
            catch (Exception e) {
                System.err.println( "FitnessWorkerThread error: " + e );
            }
        }
    }
    
    public void suspendWork() {
        isSuspended = true;
    }
    
    public synchronized void resumeWork() {
        isSuspended = false;
        notifyAll();
    }

    public synchronized void addListener( FitnessWorkerListener l ) {
        if ( listeners == null )
            listeners = new Vector();
        listeners.add( l );
    }
    
    public synchronized void removeListener( FitnessWorkerListener l ) {
        if ( listeners != null && l != null )
            listeners.remove( l );
    }
    
    public synchronized FitnessWorkerListener[] getListeners() {
        Vector list;
        FitnessWorkerListener[] retVal = null;
        if ( listeners != null ) {
            synchronized ( this ) {
                list = ( Vector ) listeners.clone();
            }
            retVal = ( FitnessWorkerListener[] ) list.toArray();
        }
        return retVal;
    }    

    private void notifyStatusChangedEvent( FitnessWorkerEvent e) {
        Vector list;
        if ( listeners != null ) {
            synchronized( this ) {
                list = ( Vector ) listeners.clone();
            }
            for ( int i = 0; i < list.size(); i++ ) {
                FitnessWorkerListener listener = ( FitnessWorkerListener ) list.elementAt( i );
                listener.statusChanged( e );
            }
        }
    }      

    private void notifyProgressMadeEvent( FitnessWorkerEvent e) {
        Vector list;
        if ( listeners != null ) {
            synchronized( this ) {
                list = ( Vector ) listeners.clone();
            }
            for ( int i = 0; i < list.size(); i++ ) {
                FitnessWorkerListener listener = ( FitnessWorkerListener ) list.elementAt( i );
                listener.progressMade( e );
            }
        }
    }      
}
