package edu.cosy.netgen.client.worker;
 
import java.awt.BorderLayout;
import javax.swing.JApplet;
import javax.swing.JFrame;
import java.net.InetAddress;
import java.rmi.RemoteException;
import java.rmi.NotBoundException;
import java.rmi.Remote;
import java.rmi.Naming;
import java.rmi.RMISecurityManager;
import java.rmi.registry.LocateRegistry;
import java.net.MalformedURLException;
import java.net.UnknownHostException;
import java.util.Properties;
import java.io.File;
import java.io.FileInputStream;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import edu.cosy.netgen.server.service.WorkingClientService;
import edu.cosy.netgen.client.service.ClientControlService;
import edu.cosy.netgen.client.service.ClientControlServiceImpl;
import edu.cosy.netgen.server.WorkingClientInfo;
import edu.cosy.netgen.client.worker.ui.WorkingClientMainPanel;
import edu.cosy.netgen.client.NetGenClient;
import edu.cosy.netgen.server.ClientControlCommand;
import edu.cosy.netgen.server.event.ServerEvent;
import edu.cosy.netgen.misc.PropertyHelper;
import edu.cosy.netgen.misc.Constants;

public class WorkingClient extends JApplet implements NetGenClient {
    private String WORKER_PROPERTIES = "worker.properties";

    private FitnessWorkerThread worker;
    private Object currentWorkingObject;    
    private PropertyHelper properties = null;
    private boolean isStandalone = true;
    private boolean isAutoConnect = true;
    private boolean isNoGUI = true;
    private WorkingClientInfo clientInfo = null; 
    private ClientControlServiceImpl controlService;
    private WorkingClientService containerService;
    private WorkingClientMainPanel gui;
    
    public WorkingClient() {
        super();
        properties = new PropertyHelper( WORKER_PROPERTIES, "netGen " + Constants.VERSION + " Working Client Properties" );
    }
        
    public WorkingClient( String server, String sPort, String id, String client, String cPort ) {
        super();
        try {
            clientInfo = new WorkingClientInfo( InetAddress.getByName( client ), Integer.parseInt( cPort ) );
            clientInfo.setAutoRunnable( false );        
            clientInfo.setServerAddress( InetAddress.getByName( server ) );
            clientInfo.setServerPort( Integer.parseInt( sPort ) );
            clientInfo.setSessionID( Long.parseLong( id ) );
        } catch ( Exception e ) {
            exit( 1 );
        }
    }

    public void exit( int state ) {
        System.out.println( "Shut down netGen Working Client ... " );
        if ( state == 1)
            System.err.println( "Abnormal termination" );
        System.out.println( "Shut down RMI server ... " );        
        try {
            if ( controlService != null )
                bindRMIObject( null, "clientControlService", false );
        } catch ( Exception e ) {
            System.err.println( "Cannot unbind RMI objects!: " + e );
        }
        disconnect();
        if ( properties != null )
            properties.close();
        if ( isStandalone )
            System.exit( state );        
    }
    
    public void init() {
        if ( isStandalone && properties != null && properties.open() ) {
            try {
                clientInfo = new WorkingClientInfo( InetAddress.getLocalHost(), Integer.parseInt( properties.getProperty( "client.port" ) ) );        
                clientInfo.setAutoRunnable( false );        
                clientInfo.setServerAddress( InetAddress.getByName( properties.getProperty( "server.host" ) ) );
                clientInfo.setServerPort( Integer.parseInt( properties.getProperty( "server.port" ) ) );
            } catch ( UnknownHostException e ) {
            }
        }      
        if ( !isNoGUI )
            getContentPane().add( gui = new WorkingClientMainPanel( this ) );        
        if ( isAutoConnect ) {
            if ( !( initializeRMI() && connect( clientInfo.getSessionID() ) ) )
                exit( 1 );
        }
    }
    
    public void destroy() {
        exit( 0 );
    }
    
    // bind/unbind an RMI object
    private boolean bindRMIObject( Remote obj, String name, boolean flag ) throws RemoteException, MalformedURLException, NotBoundException {
        boolean retVal = false;
        if ( flag ) {
            Naming.rebind( "rmi://" + clientInfo.getAddress().getHostAddress() + ":" + clientInfo.getPort() + "/" + name, obj );
            System.out.println( name + "\tbound" );    
            retVal = true;
        } else {
            Naming.unbind( "rmi://" + clientInfo.getAddress().getHostAddress() + ":" + clientInfo.getPort() + "/" + name );
            System.out.println( name + "\tunbound" );    
            retVal = true;
        }
        return retVal;
    }

    // initialize RMI connection
    private boolean initializeRMI() {
        boolean retVal = false;
        if ( clientInfo.getServerAddress() != null && clientInfo.getServerPort() != -1 ) {
            try {
                if ( controlService == null ) {
                    System.out.println( "Starting RMI server ... " );
                    
                    // create registry service
                    LocateRegistry.createRegistry( clientInfo.getPort() );
                    
                    // instanciate client service
                    controlService = new ClientControlServiceImpl( this );
                    
                    // bind object
                    retVal = bindRMIObject( controlService, "clientControlService", true );
                }
            } catch ( Exception e ) {
                System.err.println( "Cannot initialize RMI server: " + e );
                retVal = false;
            }
            try {
                containerService = ( WorkingClientService ) Naming.lookup( "rmi://" + clientInfo.getServerAddress().getHostAddress() + ":" + clientInfo.getServerPort() + "/netGenContainerService" );
                if ( containerService != null )
                    retVal = true;
            } catch ( Exception e ) {
                System.err.println( "Cannot connect to RMI server: " + e );
            }
        }
        return retVal;
    }
    
    public boolean connect() {
        if ( initializeRMI() && connect( clientInfo.getSessionID() ) ) {
            return true;
        } else
            return false;
    }
        
    public boolean connect( long sessionID ) {
        boolean retVal = false;
        try {
            if ( sessionID != -1 )
                
                // set the session ID
                retVal = containerService.connect( sessionID );
            else {
                long id = containerService.connect( clientInfo );
                if ( id != -1 ) {
                    clientInfo.setSessionID( id );
                    retVal = true;
                }
            }
        } catch ( Exception e ) {
        }
        return retVal;
    }
        
    public boolean disconnect() {
        boolean retVal = false;
        try {
            stopWorking();
            if ( containerService != null && clientInfo.getSessionID() != -1 )
                retVal = containerService.disconnect( clientInfo.getSessionID() );
            if ( retVal ) {
                clientInfo.setSessionID( -1 );
                containerService = null;
            }
        } catch ( Exception e ) {
        }
        return retVal;
    }
    
    public void setCurrentWorkingObject( Object o ) {
        currentWorkingObject = o;
        worker = null;
    }
        
    public void startWorking() {
        if ( worker == null ) {
            worker = new FitnessWorkerThread( containerService, currentWorkingObject );
            worker.start();
        }
        if ( gui != null && gui.getFitnessWorkerListener() != null )
            worker.addListener( gui.getFitnessWorkerListener() );
        worker.resumeWork();
    }
    
    public void stopWorking() {
        if ( worker != null ) {
            worker.suspendWork();
            if ( gui != null && gui.getFitnessWorkerListener() != null ) {
                worker.removeListener( gui.getFitnessWorkerListener() );
                gui.resetStatus();
            }
        }
    }
    
    public PropertyHelper getProperties() {
        return properties;
    }
    
    public WorkingClientInfo getClientInfo() {
        return clientInfo;
    }
    
    public FitnessWorkerThread getFitnessWorker() {
        return worker;
    }
    
    public boolean processCommand( ClientControlCommand cmd ) {
        boolean retVal = false;
        if ( cmd != null ) {
            if ( cmd.getCmd().equals( "start" ) ) {
                startWorking();
                return true;
            } else if ( cmd.getCmd().equals( "stop" ) ) {
                stopWorking();
                retVal = true;
            } else if ( cmd.getCmd().equals( "set" ) && cmd.getParam() != null ) {
                setCurrentWorkingObject( cmd.getParam() );
                retVal = true;
            } else if ( cmd.getCmd().equals( "quit" ) ) {
                exit( 0 );
                retVal = true;
            }
        }
        return retVal;
    }
    
    public void eventPerformed( ServerEvent e ) {
    }
    
    public static void showCommandLineHelp() {
        StringBuffer tmp = new StringBuffer( "WorkingClient [-?] [-nogui]\n\n" );
        tmp.append( "[-?]\tshow this information\n" );
        tmp.append( "[-nogui]\tdon't display the user interface\n" );
        System.out.println( tmp.toString() );
    }
        
    public static void main( String[] args ) {
        int type = 0;
        if ( args.length > 0 && args[ 0 ].equals( "-?" ) ) {
            showCommandLineHelp();
            System.exit( 0 );
        }    
        WorkingClient main = null;
        if ( args.length > 0 && args[ 0 ].equals( "-nogui" ) ) {
            if ( args.length == 6 )
                main = new WorkingClient( args[ 1 ], args[ 2 ], args[ 3 ], args[ 4 ], args[ 5 ] );
            else
                main = new WorkingClient();
            main.init();
            createNoGui( main );
        } else {
            if ( args.length == 5 )
                main = new WorkingClient( args[ 0 ], args[ 1 ], args[ 2 ], args[ 3 ], args[ 4 ] );                
            else
                main = new WorkingClient();
            main.isAutoConnect = false;
            main.isNoGUI = false;                
            main.init();
            createGui( main );
        }        
        if ( main == null ) 
        {
            System.err.println( "Cannot start netGen Working Client!" );
            System.exit( 1 );
        }
    }    
    
    private static void createGui( final WorkingClient main ) {
        JFrame mainFrame = new JFrame( "netGen " + edu.cosy.netgen.misc.Constants.VERSION + " Working Client" );
        mainFrame.addWindowListener( new WindowAdapter() {
            public void windowClosing( WindowEvent e ) {
                main.exit( 0 );
            }            
        } );
        mainFrame.getContentPane().add( main );
        mainFrame.pack();
        mainFrame.show();
    }
    
    private static void createNoGui( WorkingClient main ) {
        System.out.println( "netGen Working Client is working..." );
        System.out.print( "Type \"quit\" for exit ..." );
        try {
            byte[] command = new byte[ 4 ];
            System.in.read(command, 0, 4);
            if ( ( new String( command ) ).equals( "quit" ) )
                main.exit( 0 );
        }
        catch ( Exception e ) {
        }
    }
}
