////////////////////////////////////////////////////////////////////////////////////  
//  
//  Copyright (C) 1996 L.  Patocchi & W.Gander
//
//  This program is free  software;  you can  redistribute  it and/or  modify it
//  under the terms of the GNU General  Public  License as published by the Free
//  Software  Foundation;  either  version 2 of the License, or (at your option)
//  any later version.
//
//  This program is  distributed in the hope that it will be useful, but WITHOUT
//  ANY  WARRANTY;  without  even the implied  warranty  of  MERCHANTABILITY  or
//  FITNESS FOR A PARTICULAR  PURPOSE.  See the GNU General  Public  License for
//  more details.
//
//  You should have received a copy of the GNU General Public License along with
//  this program; if not, write to the Free Software  Foundation, Inc., 675 Mass
//  Ave, Cambridge, MA 02139, USA.
//
//  Contacts:
//  
//    Project Supervisor
//      W.Hett       hew@info.isbiel.ch
//  
//    Authors
//      W.Gander     gandw@info.isbiel.ch
//      L.Patocchi   patol@info.isbiel.ch
//
//  Documentation can be found at:
//
//      http://www.isbiel.ch/Projects/janet/index.html
//
////////////////////////////////////////////////////////////////////////////////////
//
// File : BPNgui.java
//
//////////////////////////////////////////////////////////////////
//
// 		BPNgui base class
//
//////////////////////////////////////////////////////////////////
//
//      Author:  Patocchi L.
//      Date:    10.09.1996
//      Project: jaNet
//
//		BPN graphical user interface
//       
//////////////////////////////////////////////////////////////////
//
//      date            who                                     what
//  10.09.1996  Patocchi L.                     creation
package jaNet.backprop.gui;

import java.awt.*;
import jaNet.jaNet;
import jaNet.NNeMo;
import jaNet.ModuleEntry;
import jaNet.backprop.*;

public class BPNgui extends Frame implements NNeMo,Runnable{

    Menu menuFile;
    Menu menuNeural;
    Menu menuDisplay;

	public static final String	NONE = "NONE";

	public static final String	entryPatternRefHelp = "Chose the pattern number. Shortcuts: [UP] previous pattern, [DOWN] next pattern";
	public static final String	textMonitorHelp = "Shows most important events.";
	public static final String	buttonInitNNHelp = "Sets wieghts to random values.";
	public static final String	buttonStartStopHelp = "Starts and stops the learning.";
	public static final String	buttonStepByHelp = "Learn network for more sessions.";
	public static final String	valuesScaleHelp = "Color scale reference.";
	
	// stop learning modes
	public static final int		MAXS = 0;
	public static final int		MINE = 1;
	public static final int		BOTH = 2;
	// saving BPN way when learning mode
	public static final int		SAVE_LAST = 0;
	public static final int		SAVE_BEST = 1;
	// chosing patterns way when learning mode
	public static final int		CHOSE_RAND = 0;
	public static final int		CHOSE_SEQU = 1;

    private stopRules           BPNstopRules;
    private networkStructure    BPNnetworkStructure;
    private errorGraph          BPNerrorGraph;
    private networkParameters	BPNnetworkParameters;
    private errorDialog 		BPNerrorDialog;
	private layerSettings 		BPNlayerSettings;
	private displayRules 		BPNdisplayRules;

    public BPNdraw 				canvasNeuralNet;
    public TextArea 			textMonitor;
    public Button 				buttonInitNN;
    public Button 				buttonStartStop;
    public Button 				buttonStepBy;
    public TextField 			entryStepBy;
    public TextField 			entryPatternRef;
    public Checkbox 			rbLowerError;
    public Checkbox 			rbLastCalculated;
    public Label 				labelCurrentGlobalError;
    public Label 				labelLowerGlobalError;
    public Label				labelLearningSession;
    public Label 				labelOnFoot;
    public FileDialog			saveBPN;
    public FileDialog			loadPatterns;
	public Scale 				valuesScale;

	private Color				minColor					= new Color(255,   0,   0);
	private Color				maxColor					= new Color(  0, 255, 255);

    private BPN					mainBPN;
    private BPN					undoBPN;
    
    private learningThread		currentLearningThread		= null;
    private boolean				isLearning					= false;

	// thread guard
	private Thread 				threadGuard 				= null;
	public  String				todo 						= NONE;
	
	// training parameters
	private int					stepBy						= 100;
	public int					curLearnStopMode			= BOTH;
	public int					curSaveMode					= SAVE_BEST;
	public int					maxSesStop					= 10000;
	public double				minErrStop					= 0.09;
	private learningSet			currentLearningSet			= null;
	private int					currentPattern				= 0;
	private double				minDomain					= -1.0;
	private double				maxDomain					=  1.0;
	private int					showDataFrequence			= 50;
	private int					showGraphFrequence			= 100;

	private ModuleEntry			currentMe;
	
	// behaviour
	boolean						needSave					= true;
	
//////////////////////////////////////////////////////////////////
//
//	constructor
//
//////////////////////////////////////////////////////////////////
    public BPNgui() {
		super("BPN");
		// start thread guard
		start();
	}
			
//////////////////////////////////////////////////////////////////
//
//	NNeMo interface implementors
//
//////////////////////////////////////////////////////////////////
	public void NNeMoInit(ModuleEntry me){
		currentMe = me;
		try{
			if(me.fileName != null){
				// create our main BPN reading from file
				mainBPN = new BPN(me.pathName, me.fileName);
				//System.out.println("load"+me.pathName+" "+ me.fileName);
				// create components in the Frame
				createComponents();

				setTitle(me.pathName, me.fileName);
				show();	
				needSave = false;
				canvasNeuralNet.setBPN(mainBPN);
				// set old size if exist one
				move(me.x, me.y);
				resize(me.width, me.height);
			}
		}catch(BPNException bpne){
			todo = ""+bpne;
			me.fileName = null;
		}
		if(me.fileName == null){
			this.setTitle("BPN - New");		
		
			// create buttons fields ....
			createComponents();

			// setup the minimal possible network
			int layers[] = {1,1};
			String activations[] = {"jaNet.backprop.Sigmoid"};
			try{
				mainBPN = new BPN(layers, activations);
			}catch(BPNException bpne){
				todo = ""+bpne;
			}
			
			undoBPN = null;

			mainBPN.setLearningRate(0.5);
			mainBPN.setMomentum(0.77);
			
			needSave = true;
			canvasNeuralNet.setBPN(mainBPN);
			move(50,50);
			
			me.minDomain = minDomain;
			me.maxDomain = maxDomain;
		}
		// if there was patters load them
		if(me.patternFile.compareTo("")!=0 && me.patternPath.compareTo("")!=0)
			loadPatternFile(me.patternPath, me.patternFile);
		
		show();	
	}
	public void NNeMoClose(){
		exitModule();
	}
	public void NNeMoSave(){
		selectedSave();
	}
	public boolean NNeMoNeedSave(){
		return needSave;
	}

//////////////////////////////////////////////////////////////////
//
//	gui functionality methods
//
//////////////////////////////////////////////////////////////////	
	public void refreshBPNgui(){
	
		canvasNeuralNet.setBPN(mainBPN);
		canvasNeuralNet.getGraphics().clearRect(0,0,canvasNeuralNet.size().width,canvasNeuralNet.size().height);
		canvasNeuralNet.paint(canvasNeuralNet.getGraphics());
		
		if(BPNstopRules != null) 			BPNstopRules.hide();
		if(BPNnetworkStructure != null) 	BPNnetworkStructure.hide();
		if(BPNnetworkParameters != null) 	BPNnetworkParameters.hide();
		if(BPNerrorDialog != null) 			BPNerrorDialog.hide();
		BPNerrorGraph.showElements();
		showPattern();
	}
	
    public void showPattern(int pattern){
		if(mainBPN != null && currentLearningSet != null){
			if(pattern <0 || pattern > currentLearningSet.getSize()) return;
			try{
				mainBPN.propagate(currentLearningSet.inputs[pattern].vector);
			}catch(BPNException bpne){
				todo = ""+bpne;
				return;
			}
			canvasNeuralNet.updateValues(mainBPN.getBPNdescriptor());
		}
    }
    
    public void showPattern(){
		showPattern(currentPattern);
		BPNerrorGraph.showElements();
    }
	public void paint(Graphics g){
		currentMe.width = size().width;
		currentMe.height = size().height;
		super.paint(g);
	}
 
	public void exitModule(){
		// hide all sub-windows
		if(BPNstopRules != null) 			BPNstopRules.hide();
		if(BPNnetworkStructure != null) 	BPNnetworkStructure.hide();
		if(BPNnetworkParameters != null) 	BPNnetworkParameters.hide();
		if(BPNerrorDialog != null) 			BPNerrorDialog.hide();
		if(BPNerrorGraph != null) 			BPNerrorGraph.hide();

		hide();         // hide the Frame
	    //dispose();    // tell windowing system to free resources
	    if(currentLearningThread != null) currentLearningThread.stop();
	    if(threadGuard != null) threadGuard.stop();
	    needSave = false;
	    //System.exit(0); // exit
	}

    public boolean handleEvent(Event event) {
		//System.out.println(""+event.id+" "+event.target);
		if(event.target == entryPatternRef){
			putOnFooter(entryPatternRefHelp);
		}else
		if(event.target == textMonitor){
			putOnFooter(textMonitorHelp);
		}else
		if(event.target == valuesScale){
			putOnFooter(valuesScaleHelp);
		}

     	if (event.id == Event.ACTION_EVENT && event.target == buttonInitNN) {
				// initialise NN
				try{
					mainBPN.randomize();
					putOnMonitor("Weights has been randomized ( min = "+mainBPN.getInitMin()+" , max = "+mainBPN.getInitMax()+" )\n");
					refreshBPNgui();
    	    	}catch(BPNException bpne){
					todo = ""+bpne;
    	    	}
    	    	return true;
    	}
    	else
     	if (event.id == Event.ACTION_EVENT && event.target == buttonStartStop) {
				if(buttonStartStop.getLabel().compareTo("Start")==0){
					selectedStartLearning();
				}else{
					selectedStopLearning();
				}
    	    	return true;
    	}
    	else
     	if (event.id == Event.ACTION_EVENT && event.target == buttonStepBy) {
				selectedStepBy();
    	    	return true;
    	}else
    	if (event.id == Event.KEY_RELEASE && event.target == entryStepBy) {
				String entry = entryStepBy.getText();
				try{
					stepBy = Integer.parseInt(entry);
					//putOnMonitor("step by = "+stepBy+"\n");
				}catch(NumberFormatException nfe){
					if(entry.compareTo("") != 0){
						todo = ""+nfe;
					}else{
						stepBy = 0;
						//putOnMonitor("step by = "+stepBy+"\n");
					}
				}
    	    	return true;
    	}else
    	if (event.id == Event.KEY_RELEASE && event.target == entryPatternRef) {
			String entry = entryPatternRef.getText();
			int value;
			try{
				value = Integer.parseInt(entry);
				//putOnMonitor("step by = "+stepBy+"\n");
			}catch(NumberFormatException nfe){
				if(entry.compareTo("") != 0){
					todo = ""+nfe;
				}
				return true;
			}
			if(value < 0 || value > currentLearningSet.getSize()-1) return true;
			currentPattern = value;
			showPattern();
			return true;
    	}else
    	if (event.id == Event.KEY_ACTION_RELEASE && event.target == entryPatternRef){
			String entry = entryPatternRef.getText();
			int value;
			try{
				value = Integer.parseInt(entry);
				//putOnMonitor("step by = "+stepBy+"\n");
			}catch(NumberFormatException nfe){
				if(entry.compareTo("") != 0){
					todo = ""+nfe;
				}
				return true;
			}
			if(event.key == Event.UP){
				if(value+1 > currentLearningSet.getSize()-1) return true;
				currentPattern = value+1;
			}else
			if(event.key == Event.DOWN){
				if(value-1 < 0) return true;
				currentPattern = value-1;
			}
			entryPatternRef.setText(""+currentPattern);
			showPattern();
			return true;
    	}else
	   	if (event.id == Event.WINDOW_MOVED) {
			currentMe.x = location().x;
			currentMe.y = location().y;
            return true;
    	}else
	   	if (event.id == Event.WINDOW_DESTROY) {
			exitModule();
            return true;
    	}
    	return super.handleEvent(event);
    }

    public boolean action(Event event, Object arg) {
		if (event.target instanceof MenuItem) {
			String label = (String) arg;
			if (label.equalsIgnoreCase("Error Graph...")) {
				selectedErrorGraph();
				return true;
			} else if (label.equalsIgnoreCase("Parameters ...")) {
				selectedNetworkParameters();
				return true;
			} else if (label.equalsIgnoreCase("Stop Rules...")) {
				selectedStopRules();
				return true;
			} else if (label.equalsIgnoreCase("Load Patterns ...")) {
				selectedLoadPatterns();
				return true;
			} else if (label.equalsIgnoreCase("Close Module")) {
				selectedCloseModule();
				return true;
			} else if (label.equalsIgnoreCase("Save As...")) {
				selectedSaveAs();
				return true;
			} else if (label.equalsIgnoreCase("Save")) {
				selectedSave();
				return true;
			} else if (label.equalsIgnoreCase("Display frequency...")) {
				selectedDisplayFrequences();
				return true;
			}
		}
		return super.action(event, arg);
    }
    public void selectedSave() {
		if(isLearning) return;
		if(currentMe.pathName==null || currentMe.fileName==null){
			selectedSaveAs();
			return;
		}
		// saves current state !
		if(saveModule(currentMe.pathName, currentMe.fileName))/* it's OK */;
    }
    public void selectedSaveAs() {
		if(isLearning) return;
		if(currentMe.pathName!=null)
			saveBPN.setDirectory(currentMe.pathName);
		if(currentMe.fileName!=null)
			saveBPN.setFile(currentMe.fileName);
		
		saveBPN.show();
		String file = saveBPN.getFile();
		if(file==null) return;
		if(file.endsWith("*.*")) file = file.substring(file.length()-3,file.length());
		// do it remains something ?
		if(file.length() == 0) return;
		
		if(saveModule(saveBPN.getDirectory(), file)){
			currentMe.pathName = "."; //saveBPN.getDirectory();
			currentMe.fileName = file;
			setTitle(currentMe.pathName,currentMe.fileName);
		}
    }
    public void selectedCloseModule() {
		if(isLearning) return;
		exitModule();
    }
    public void selectedLoadPatterns() {
		if(isLearning) return;
		String				patternfile;
		String				patternpath;

		if(currentMe.patternPath != null && currentMe.patternPath.compareTo("")!=0)
			loadPatterns.setDirectory(currentMe.patternPath);
		loadPatterns.show();
		patternfile  = loadPatterns.getFile();
		if(patternfile == null) return;
	
		patternpath = loadPatterns.getDirectory();
		loadPatternFile(patternpath, patternfile);
	}
    
    public void selectedStartLearning() {
		try{
			if(currentLearningThread == null){
				currentLearningThread = new 
					learningThread(this, mainBPN, currentLearningSet, 
									curLearnStopMode, maxSesStop, minErrStop,
									curSaveMode, CHOSE_RAND, showDataFrequence, showGraphFrequence);
				currentLearningThread.start();
				buttonStartStop.setLabel("Stop");
				isLearning = true;
			}
		}catch(BPNException bpne){
			todo = ""+bpne;
			return;
		}
	}
	public void selectedStepBy(){
		try{
			if(currentLearningThread == null){
				currentLearningThread = new 
					learningThread(this, mainBPN, currentLearningSet, 
									MAXS, stepBy, 0.0,
									curSaveMode, CHOSE_RAND, showDataFrequence, showGraphFrequence);
				currentLearningThread.start();
				buttonStartStop.setLabel("Stop");
				isLearning = true;
			}else{
				todo = "BPN Neural Network is already in a learning session.";
				return;
			}
		}catch(BPNException bpne){
			todo = ""+bpne;
			return;
		}
	}
    public void selectedStopLearning() {
		if(currentLearningThread != null){
			currentLearningThread.stopNow();
			currentLearningThread = null;
			System.gc();
		}
		isLearning = false;
		buttonStartStop.setLabel("Start");
		refreshBPNgui();
    }
    public void selectedStopRules() {
        if(BPNstopRules == null) BPNstopRules = new stopRules(this);
        BPNstopRules.show(curLearnStopMode, maxSesStop, minErrStop);
    }
    public void selectedNetworkParameters() {
		if(isLearning) return;
        if(BPNnetworkParameters == null) BPNnetworkParameters = new networkParameters(this);
        BPNnetworkParameters.show(mainBPN, minDomain, maxDomain);
    }
    public void selectedErrorGraph() {
        if(BPNerrorGraph.isVisible()){
			BPNerrorGraph.hide( );
        }else{
			BPNerrorGraph.show(minErrStop);
		}
    }
    public void  selectedDisplayFrequences(){
		if(BPNdisplayRules == null) BPNdisplayRules = new displayRules(this);
        BPNdisplayRules.show(showDataFrequence, showGraphFrequence);
    }
    
//////////////////////////////////////////////////////////////////
//
//	network structure modifiers
//
//////////////////////////////////////////////////////////////////
	public void addLayerBefore(int selectedLayer){
		if(isLearning) return;
		if(selectedLayer==0){
			todo = "Cannot create layers before Input Layer.";
			return;
		}
		BPNdescriptor bpnd = mainBPN.getBPNdescriptor();
		
		// allocate enough mem
		int newLayerDef[] 		= new int[bpnd.layerDef.length+1];
		String newActFnDef[] 	= new String[bpnd.layerDef.length];
		double newBiasDef[] 	= new double[bpnd.layerDef.length+1];
	
		for(int i=0,oldref=0; i<bpnd.layerDef.length+1;i++){
			if(i == selectedLayer){
				newLayerDef[i] 		= 1;  // set default layer lenght
				newBiasDef[i] 		= 0.0;// set default bias value
				// copy from layer reference (which was poppedup) 
				// NOTE: the use of offs to reference right layer
				newActFnDef[i-1] 	= bpnd.activationFnDef[i-1];
			}else{
				// WARNING :lost of details !
				newBiasDef[i] = bpnd.layerBias1[oldref];
				newLayerDef[i] = bpnd.layerDef[oldref];
				if(i!=0) newActFnDef[i-1]= bpnd.activationFnDef[oldref-1];
				oldref++;
			}
		}
		createNewMainBPN(newLayerDef, newActFnDef, newBiasDef);
	}
	public void addLayerAfter(int selectedLayer){
		BPNdescriptor bpnd = mainBPN.getBPNdescriptor();
		if(selectedLayer==bpnd.layerDef.length-1){
			todo = "Cannot create layers after Output Layer.";
			return;
		}
		// as stupid it could seem ...
		addLayerBefore(selectedLayer + 1);
	}
	public void deleteLayer(int selectedLayer){
		if(isLearning) return;
		BPNdescriptor bpnd = mainBPN.getBPNdescriptor();
		if(selectedLayer==0){
			todo = "Cannot delete Input Layer.";
			return;
		}
		if(selectedLayer==bpnd.layerDef.length-1){
			todo = "Cannot delete Output Layer.";
			return;
		}
		// allocate enough mem
		int newLayerDef[] 		= new int[bpnd.layerDef.length-1];
		double newBiasDef[] 	= new double[bpnd.layerDef.length-1];
		String newActFnDef[] 	= new String[bpnd.layerDef.length-2];
		
		for(int i=0,newref=0; i<bpnd.layerDef.length;i++){
			if(i != selectedLayer){
				newLayerDef[newref] = bpnd.layerDef[i];
				// WARNING :lost of details !
				newBiasDef[newref] = bpnd.layerBias1[i];
				if(newref!=0) newActFnDef[newref-1]= bpnd.activationFnDef[i-1];
				newref++;
			}
		}
		createNewMainBPN(newLayerDef, newActFnDef, newBiasDef);
	}
	public void editLayer(int selectedLayer){
		BPNdescriptor bpnd = mainBPN.getBPNdescriptor();

	    if(BPNlayerSettings == null)BPNlayerSettings = new layerSettings(this);
		BPNlayerSettings.show(bpnd, selectedLayer, "Layer "+selectedLayer);
	}
	public void createNewMainBPN(int newLayerDef[], String newActFnDef[], double newBias[]){
		if(isLearning) return;
		double tempMomentum 	= mainBPN.getMomentum(); 
		double tempLearningRate = mainBPN.getLearningRate();
		double tempInitMin		= mainBPN.getInitMin();
		double tempInitMax		= mainBPN.getInitMax();
		BPNdescriptor bpnd		= mainBPN.getBPNdescriptor();
		
		BPN tempBPN = null;
		
		// if input or output layer change size the patterns become obsolete
		if(currentLearningSet != null && (newLayerDef[0] != bpnd.layerDef[0] || 
		   newLayerDef[newLayerDef.length-1] != bpnd.layerDef[bpnd.layerDef.length-1])){
			currentLearningSet = null;
			entryPatternRef.setText("0");
			entryPatternRef.disable();
			valuesScale.setScaleIsOn(false);
			putOnMonitor("Input or Output layer size changed, patterns are obsolete.\n");
		}
		bpnd = null;
		
		// Create a new BPN with new descriptors !
		try{
			tempBPN = new BPN(newLayerDef, newActFnDef);
		}catch(BPNException bpne){
			todo = ""+bpne;
			return;
		}
		if(tempBPN==null){
			todo = "Something gone wrong on creating new NN.";
			return;
		}

		// free some memory
		undoBPN = mainBPN;
		System.gc();
		// installing new one
		mainBPN = tempBPN;
		// refreshing data and graphics in BPNgui
		refreshBPNgui();
		// restoring old NN settings
		mainBPN.setMomentum(tempMomentum); 
		mainBPN.setLearningRate(tempLearningRate);
		mainBPN.setInitMin(tempInitMin);
		mainBPN.setInitMax(tempInitMax);
		// randomizing weigths
		try{
			mainBPN.randomize();
		}catch(BPNException bpne){
			todo = ""+bpne;
		}
		for(int i=0; i<newLayerDef.length; i++)
			setBias(i, newBias[i]);
		needSave = true;
	}
	public void undoAction(int selectedLayer){
//		if(undoBPN == null){
//			todo = "Nothing to undo.";
//			return;
//		}		
//		BPN tempBPN = undoBPN;
//		// perform swap
//		undoBPN = mainBPN;
//		mainBPN = tempBPN;
//
//		// refreshing data and graphics in BPNgui
//		refreshBPNgui();
//		putOnMonitor("Undo last action.\n");
	}

//////////////////////////////////////////////////////////////////
//
//	Callbacks from windows
//
//////////////////////////////////////////////////////////////////
	public void setDomainRange(double min, double max){
		minDomain = min;
		maxDomain = max;
		valuesScale.setDomainValue(min, max);
		if(currentLearningSet != null){
			currentLearningSet.scaleDataTo(min,max);
			refreshBPNgui();
		}
		currentMe.minDomain = minDomain;
		currentMe.maxDomain = maxDomain;
	}
	public void setDisplayFrequences(int dataFreq, int graphFreq){
		showDataFrequence  = dataFreq;
		showGraphFrequence = graphFreq;
	}
	public void setMinimalErrorStop(double val){
		minErrStop = val;
		BPNerrorGraph.setMinLimit(minErrStop);
	}
	public void setMaximalSessionStop(int val){
		maxSesStop = val;
	}
	public void setCurrentError(double err){
		labelCurrentGlobalError.setText(""+err);
		BPNerrorGraph.putElement(err);
	}
	public void setCurrentSession(int val){
		labelLearningSession.setText(""+val);
	}
	public void setLowerGlobalError(double val){
		labelLowerGlobalError.setText(""+val);
		labelCurrentGlobalError.setText(""+val);
		BPNerrorGraph.putElement(val);
	}
//////////////////////////////////////////////////////////////////
//
//	network values modifiers
//
//////////////////////////////////////////////////////////////////
	public void setBias(int layer, double value){
		if(isLearning) return;
		try{
			mainBPN.setBias(layer, value);
			if(currentLearningSet != null)
				refreshBPNgui();
		}catch(BPNException bpne){
			todo = ""+bpne;
		}
		needSave = true;
	}
	public void setActivationFnClass(int layer, String ActFn){
		if(isLearning) return;
		try{
			mainBPN.setActivationFnClass(layer, ActFn);
			if(currentLearningSet != null)
				refreshBPNgui();
		}catch(BPNException bpne){
			todo = ""+bpne;
		}
		needSave = true;
	}
//////////////////////////////////////////////////////////////////
//
//	network I/O
//
//////////////////////////////////////////////////////////////////
	public boolean saveModule(String path, String file){
		try{
			mainBPN.saveNeuralNetwork(path,file);
		}catch(BPNException bpne){
			todo = ""+bpne;
			return false;
		}
		needSave = false;
		putOnMonitor("Module saved to "+path+file+"\n");
		return true;
	}
	
	public void loadPatternFile(String patternpath, String patternfile){
		learningSet			tempLearningSet;
		BPNdescriptor		bpnd = mainBPN.getBPNdescriptor();
				
		try{
			tempLearningSet = new learningSet(patternpath,patternfile, bpnd.layerDef[0], bpnd.layerDef[bpnd.layerDef.length-1], minDomain, maxDomain );
		}catch(BPNException bpne){
			todo = ""+bpne;
			return;
		}
		currentMe.patternPath = "."; //patternpath;
		currentMe.patternFile = patternfile;
		// enable components
		entryPatternRef.enable();
		valuesScale.setScaleIsOn(true);
		
		currentLearningSet = tempLearningSet;
		showPattern();
		putOnMonitor("successfully loaded "+currentLearningSet.getSize()+" patterns.\n");
    }

//////////////////////////////////////////////////////////////////
//
//	Monitoring & footering
//
//////////////////////////////////////////////////////////////////
	public void putOnMonitor(String msg){
		textMonitor.appendText(msg);
	}
	public void putOnFooter(String msg){
    	labelOnFoot.setText(msg);
	}
	

//////////////////////////////////////////////////////////////////
//
//	thread guard starter, stopper and body
//
//////////////////////////////////////////////////////////////////
	public void start() { 
	  if(threadGuard == null){
			threadGuard = new Thread(this, "BPNgui guard");
			threadGuard.start();
		}
	}

	public void stop(){
		threadGuard.stop();
		threadGuard = null;
	}
	
	public void run(){
		Runtime rt = Runtime.getRuntime();
		long mem1,mem2;
		while(true){			
			// Call Garbage Collect
			//System.gc();
			if(todo.compareTo(NONE) != 0){
				BPNerrorDialog = new errorDialog(this,todo);
				BPNerrorDialog.show();
				
				todo = NONE;
			}
			mem1 = rt.totalMemory()-rt.freeMemory();
			// never try to free some memory ...
			System.gc();
			mem2 = rt.totalMemory()-rt.freeMemory();
			//putOnFooter("Used Memory "+mem2+" of total "+rt.totalMemory()+" from last GC saved "+(mem2-mem1));
			// wait a tenth of a second, then draw it again!
			try {Thread.sleep(500);} catch (InterruptedException e){};
		}
	}
//////////////////////////////////////////////////////////////////
//
//	Window constructors
//
//////////////////////////////////////////////////////////////////

	public void setTitle(String path, String name){
		this.setTitle("BPN - "+path+name);
	}

    private void reset(GridBagConstraints con) {
        con.gridx = GridBagConstraints.RELATIVE;
        con.gridy = GridBagConstraints.RELATIVE;
        con.gridwidth = 1;
        con.gridheight = 1;

        con.weightx = 0;
        con.weighty = 0;
        con.anchor = GridBagConstraints.CENTER;
        con.fill = GridBagConstraints.NONE;

        con.insets = new Insets(0, 0, 0, 0);
        con.ipadx = 0;
        con.ipady = 0;
    }
	private void createComponents(){  
        int w=0, h=0;
        Separator separatorV1;
        Separator separatorV2;
        Separator separatorH1;
        Separator separatorH2;
        Separator separatorH3;
        Separator separatorH4;
        Separator separatorH5;
        Separator separatorH6;
        Label label_2;
        Label label_3;
        Label label_4;
        Label label_5;
        Label label_8;
        Label label_10;
        Label label_11;
        Label label_12;
        Label label_13;
		
		// offscreen Creations
		saveBPN 		= new FileDialog(this, "Save BPN File", FileDialog.SAVE);
		loadPatterns 	= new FileDialog(this, "Load Pattern File ", FileDialog.LOAD);
		BPNerrorGraph 	= new errorGraph();
		
        // main panel
    	GridBagLayout grid = new GridBagLayout();
    	int rowHeights[] = {0,5,111,5,20,30,5,20,25,25,5,20,15,15,15,5,20,30,5,20};
		int columnWidths[] = {0,300,5,50,5,75,75,35,75,5};
	    //int columnWidths[] = {0,300,5,75,75,35,75,5,40};
	    double rowWeights[] = {0.0,0.0,0.0,0.0,0.0,0.0,1.0,0.0,0.0,0.0,1.0,0.0,0.0,0.0,0.0,1.0,0.0,0.0,0.0,0.01};
		double columnWeights[] = {0.0,1.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0};
	    grid.rowHeights = rowHeights;
    	grid.columnWidths = columnWidths;
    	grid.rowWeights = rowWeights;
    	grid.columnWeights = columnWeights;

		w = 0;
		h = 0;
		for(int i=0;i<rowHeights.length;i++) h += rowHeights[i];
	    for(int i=0;i<columnWidths.length;i++) w += columnWidths[i];
        resize(insets().left + insets().right+w+50, insets().top + insets().bottom + h +70);

        this.setBackground(Color.lightGray);

        CheckboxGroup keepWeightsGroup = new CheckboxGroup();

    	separatorV1 = new Separator(Separator.VERTICAL);
    	separatorV1.perSide = 0;
    	separatorV1.rise = Separator.IN;
    	this.add(separatorV1);

        separatorH3 = new Separator(Separator.HORIZONTAL);
    	separatorH3.perSide = 0;
    	separatorH3.rise = Separator.IN;
    	this.add(separatorH3);

    	canvasNeuralNet = new BPNdraw(this);
    	this.add(canvasNeuralNet);

    	textMonitor = new TextArea(1,1);
    	textMonitor.setEditable(false);
    	this.add(textMonitor);

    	separatorH1 = new Separator(Separator.HORIZONTAL);
    	separatorH1.perSide = 0;
    	separatorH1.rise = Separator.IN;
    	this.add(separatorH1);

    	label_2 = new Label();
    	label_2.setFont(new Font("Helvetica",Font.BOLD , 12));
    	label_2.setText("Network simulation");
    	this.add(label_2);

    	separatorV2 = new Separator(Separator.VERTICAL);
    	separatorV2.perSide = 0;
    	separatorV2.rise = Separator.IN;
    	this.add(separatorV2);

    	label_3 = new Label();
    	label_3.setFont(new Font("Helvetica",Font.PLAIN , 12));
    	label_3.setText("Scale");
    	this.add(label_3);

    	buttonInitNN = new Button();
    	buttonInitNN.setLabel("Init NN");
    	this.add(buttonInitNN);

    	buttonStartStop = new Button();
    	buttonStartStop.setLabel("Start");
    	this.add(buttonStartStop);

    	buttonStepBy = new Button();
    	buttonStepBy.setLabel("Step by");
    	this.add(buttonStepBy);

    	entryStepBy = new TextField(8);
    	entryStepBy.setText(""+stepBy);
    	this.add(entryStepBy);

    	entryPatternRef = new TextField(8);
    	entryPatternRef.setText(""+currentPattern);
    	entryPatternRef.disable();
    	this.add(entryPatternRef);

    	valuesScale = new Scale(this, minDomain, maxDomain, minColor, maxColor);
    	this.add(valuesScale);
    	
    	// give this scale to ...
    	canvasNeuralNet.setScale(valuesScale);

    	separatorH4 = new Separator(Separator.HORIZONTAL);
    	separatorH4.perSide = 0;
    	separatorH4.rise = Separator.IN;
    	this.add(separatorH4);

    	label_4 = new Label();
    	label_4.setFont(new Font("Helvetica",Font.BOLD , 12));
    	label_4.setText("Keep weights...");
    	this.add(label_4);

    	rbLowerError = new Checkbox("which gives the lowest global error",keepWeightsGroup, (curSaveMode == SAVE_BEST));
    	rbLowerError.setFont(new Font("Helvetica",Font.PLAIN , 12));
    	this.add(rbLowerError);

    	rbLastCalculated = new Checkbox("that have been calculated the last",keepWeightsGroup,(curSaveMode == SAVE_LAST));
    	rbLastCalculated.setFont(new Font("Helvetica",Font.PLAIN , 12));
    	this.add(rbLastCalculated);

    	separatorH5 = new Separator(Separator.HORIZONTAL);
    	separatorH5.perSide = 0;
    	separatorH5.rise = Separator.IN;
    	this.add(separatorH5);

    	label_5 = new Label();
    	label_5.setFont(new Font("Helvetica",Font.BOLD , 12));
    	label_5.setText("Continous data");
    	this.add(label_5);

    	label_12 = new Label();
    	label_12.setFont(new Font("Helvetica",Font.PLAIN , 12));
    	label_12.setText("current global error");
    	this.add(label_12);

    	labelCurrentGlobalError = new Label();
    	labelCurrentGlobalError.setFont(new Font("Helvetica",Font.BOLD , 12));
    	this.add(labelCurrentGlobalError);

    	label_8 = new Label();
    	label_8.setFont(new Font("Helvetica",Font.PLAIN , 12));
    	label_8.setText("lowest global error");
    	this.add(label_8);

    	labelLowerGlobalError = new Label();
    	labelLowerGlobalError.setFont(new Font("Helvetica",Font.BOLD , 12));
    	this.add(labelLowerGlobalError);

    	label_10 = new Label();
    	label_10.setFont(new Font("Helvetica",Font.PLAIN , 12));
    	label_10.setText("learning sessions");
    	this.add(label_10);

    	labelLearningSession = new Label();
    	labelLearningSession.setFont(new Font("Helvetica",Font.BOLD , 12));
    	this.add(labelLearningSession);

    	separatorH6 = new Separator(Separator.HORIZONTAL);
    	separatorH6.perSide = 0;
    	separatorH6.rise = Separator.IN;
    	this.add(separatorH6);

    	label_13 = new Label();
    	label_13.setFont(new Font("Helvetica",Font.BOLD , 12));
    	label_13.setText("Patterns");
    	this.add(label_13);

    	label_11 = new Label();
    	label_11.setFont(new Font("Helvetica",Font.PLAIN , 12));
    	label_11.setText("Display pattern :");
    	this.add(label_11);

    	separatorH2 = new Separator(Separator.HORIZONTAL);
    	separatorH2.perSide = 0;
    	separatorH2.rise = Separator.IN;
    	this.add(separatorH2);

    	labelOnFoot = new Label();
    	labelOnFoot.setFont(new Font("Helvetica",Font.PLAIN , 12));
    	this.add(labelOnFoot);

    	// Geometry management
    	GridBagConstraints con = new GridBagConstraints();
    	reset(con);
    	con.gridx = 1;
    	con.gridy = 1;
    	con.gridwidth = 8;
    	con.anchor = GridBagConstraints.CENTER;
    	con.fill = GridBagConstraints.BOTH;
    	grid.setConstraints(separatorH3, con);

    	reset(con);
    	con.gridx = 1;
    	con.gridy = 2;
    	con.gridheight = 16;
    	con.anchor = GridBagConstraints.CENTER;
    	con.fill = GridBagConstraints.BOTH;
    	grid.setConstraints(canvasNeuralNet, con);

    	reset(con);
    	con.gridx = 2;
    	con.gridy = 2;
    	con.gridheight = 16;
    	con.anchor = GridBagConstraints.CENTER;
    	con.fill = GridBagConstraints.BOTH;
    	grid.setConstraints(separatorV1, con);

    	reset(con);
    	con.gridx = 3;
    	con.gridy = 2;
    	con.gridwidth = 7;
    	con.anchor = GridBagConstraints.CENTER;
    	con.fill = GridBagConstraints.BOTH;
    	grid.setConstraints(textMonitor, con);

    	reset(con);
    	con.gridx = 3;
    	con.gridy = 3;
    	con.gridwidth = 7;
    	con.anchor = GridBagConstraints.CENTER;
    	con.fill = GridBagConstraints.BOTH;
    	grid.setConstraints(separatorH1, con);

    	reset(con);
    	con.gridx = 5;
    	con.gridy = 4;
    	con.gridwidth = 4;
    	con.anchor = GridBagConstraints.CENTER;
    	con.fill = GridBagConstraints.HORIZONTAL;
    	grid.setConstraints(label_2, con);

    	reset(con);
    	con.gridx = 4;
    	con.gridy = 4;
    	con.gridheight = 14;
    	con.anchor = GridBagConstraints.CENTER;
    	con.fill = GridBagConstraints.BOTH;
    	grid.setConstraints(separatorV2, con);

    	reset(con);
    	con.gridx = 3;
    	con.gridy = 4;
    	con.anchor = GridBagConstraints.CENTER;
    	con.fill = GridBagConstraints.HORIZONTAL;
    	grid.setConstraints(label_3, con);

    	reset(con);
    	con.gridx = 5;
    	con.gridy = 5;
    	con.anchor = GridBagConstraints.CENTER;
    	con.fill = GridBagConstraints.HORIZONTAL;
    	grid.setConstraints(buttonInitNN, con);

    	reset(con);
    	con.gridx = 6;
    	con.gridy = 5;
    	con.anchor = GridBagConstraints.CENTER;
    	con.fill = GridBagConstraints.HORIZONTAL;
    	grid.setConstraints(buttonStartStop, con);

    	reset(con);
    	con.gridx = 7;
    	con.gridy = 5;
    	con.anchor = GridBagConstraints.CENTER;
    	con.fill = GridBagConstraints.HORIZONTAL;
    	grid.setConstraints(buttonStepBy, con);

    	reset(con);
    	con.gridx = 8;
    	con.gridy = 5;
    	con.anchor = GridBagConstraints.CENTER;
    	con.fill = GridBagConstraints.HORIZONTAL;
    	grid.setConstraints(entryStepBy, con);

    	reset(con);
    	con.gridx = 3;
    	con.gridy = 5;
    	con.gridheight = 13;
    	con.anchor = GridBagConstraints.CENTER;
    	con.fill = GridBagConstraints.BOTH;
    	grid.setConstraints(valuesScale, con);

    	reset(con);
    	con.gridx = 5;
    	con.gridy = 6;
    	con.gridwidth = 5;
    	con.anchor = GridBagConstraints.CENTER;
    	con.fill = GridBagConstraints.BOTH;
    	grid.setConstraints(separatorH4, con);

    	reset(con);
    	con.gridx = 5;
    	con.gridy = 7;
    	con.gridwidth = 4;
    	con.anchor = GridBagConstraints.CENTER;
    	con.fill = GridBagConstraints.HORIZONTAL;
    	grid.setConstraints(label_4, con);

    	reset(con);
    	con.gridx = 6;
    	con.gridy = 8;
    	con.gridwidth = 3;
    	con.anchor = GridBagConstraints.WEST;
    	con.fill = GridBagConstraints.HORIZONTAL;
    	grid.setConstraints(rbLowerError, con);

    	reset(con);
    	con.gridx = 6;
    	con.gridy = 9;
    	con.gridwidth = 3;
    	con.anchor = GridBagConstraints.WEST;
    	con.fill = GridBagConstraints.HORIZONTAL;
    	grid.setConstraints(rbLastCalculated, con);

    	reset(con);
    	con.gridx = 5;
    	con.gridy = 10;
    	con.gridwidth = 5;
    	con.anchor = GridBagConstraints.CENTER;
    	con.fill = GridBagConstraints.BOTH;
    	grid.setConstraints(separatorH5, con);

    	reset(con);
    	con.gridx = 5;
    	con.gridy = 11;
    	con.gridwidth = 4;
    	con.anchor = GridBagConstraints.CENTER;
    	con.fill = GridBagConstraints.HORIZONTAL;
    	grid.setConstraints(label_5, con);

    	reset(con);
    	con.gridx = 5;
    	con.gridy = 12;
    	con.gridwidth = 2;
    	con.anchor = GridBagConstraints.CENTER;
    	con.fill = GridBagConstraints.HORIZONTAL;
    	grid.setConstraints(label_12, con);

    	reset(con);
    	con.gridx = 7;
    	con.gridy = 12;
    	con.gridwidth = 2;
    	con.anchor = GridBagConstraints.CENTER;
    	con.fill = GridBagConstraints.HORIZONTAL;
    	grid.setConstraints(labelCurrentGlobalError, con);

    	reset(con);
    	con.gridx = 5;
    	con.gridy = 13;
    	con.gridwidth = 2;
    	con.anchor = GridBagConstraints.CENTER;
    	con.fill = GridBagConstraints.HORIZONTAL;
    	grid.setConstraints(label_8, con);

    	reset(con);
    	con.gridx = 7;
    	con.gridy = 13;
    	con.gridwidth = 2;
    	con.anchor = GridBagConstraints.CENTER;
    	con.fill = GridBagConstraints.HORIZONTAL;
    	grid.setConstraints(labelLowerGlobalError, con);

    	reset(con);
    	con.gridx = 5;
    	con.gridy = 14;
    	con.gridwidth = 2;
    	con.anchor = GridBagConstraints.CENTER;
    	con.fill = GridBagConstraints.HORIZONTAL;
    	grid.setConstraints(label_10, con);

    	reset(con);
    	con.gridx = 7;
    	con.gridy = 14;
    	con.gridwidth = 2;
    	con.anchor = GridBagConstraints.CENTER;
    	con.fill = GridBagConstraints.HORIZONTAL;
    	grid.setConstraints(labelLearningSession, con);

    	reset(con);
    	con.gridx = 5;
    	con.gridy = 15;
    	con.gridwidth = 5;
    	con.anchor = GridBagConstraints.CENTER;
    	con.fill = GridBagConstraints.BOTH;
    	grid.setConstraints(separatorH6, con);

    	reset(con);
    	con.gridx = 5;
    	con.gridy = 16;
    	con.gridwidth = 4;
    	con.anchor = GridBagConstraints.CENTER;
    	con.fill = GridBagConstraints.HORIZONTAL;
    	grid.setConstraints(label_13, con);

    	reset(con);
    	con.gridx = 5;
    	con.gridy = 17;
    	con.gridwidth = 2;
    	con.anchor = GridBagConstraints.CENTER;
    	con.fill = GridBagConstraints.HORIZONTAL;
    	grid.setConstraints(label_11, con);

    	reset(con);
    	con.gridx = 7;
    	con.gridy = 17;
    	con.gridwidth = 2;
    	con.anchor = GridBagConstraints.CENTER;
    	con.fill = GridBagConstraints.HORIZONTAL;
    	grid.setConstraints(entryPatternRef, con);

    	reset(con);
    	con.gridx = 1;
    	con.gridy = 18;
    	con.gridwidth = 9;
    	con.anchor = GridBagConstraints.CENTER;
    	con.fill = GridBagConstraints.BOTH;
    	grid.setConstraints(separatorH2, con);

    	reset(con);
    	con.gridx = 1;
    	con.gridy = 19;
    	con.gridwidth = 8;
    	con.anchor = GridBagConstraints.CENTER;
    	con.fill = GridBagConstraints.HORIZONTAL;
    	grid.setConstraints(labelOnFoot, con);


    	// Resize behavior management and parent heirarchy
    	setLayout(grid);
		
		// add menubar and all menus
        MenuBar mb = new MenuBar();
        menuFile = new Menu("File");
        menuFile.add(new MenuItem("Save"));
        menuFile.add(new MenuItem("Save As..."));
        menuFile.addSeparator();
        menuFile.add(new MenuItem("Close Module"));
        mb.add(menuFile);
        menuNeural = new Menu("Network");
        menuNeural.add(new MenuItem("Load Patterns ..."));
        menuNeural.addSeparator();
        menuNeural.add(new MenuItem("Stop Rules..."));
        menuNeural.addSeparator();
        menuNeural.add(new MenuItem("Parameters ..."));
        mb.add(menuNeural);
        menuDisplay = new Menu("Display");
        menuDisplay.add(new MenuItem("Error Graph..."));
        menuDisplay.add(new MenuItem("Display frequency..."));
        mb.add(menuDisplay);
        setMenuBar(mb);
        
        addNotify();
        pack();
    }

//////////////////////////////////////////////////////////////////
//
//	tester / debugger
//
//////////////////////////////////////////////////////////////////	
//	public static void main(String args[]) {
//        NNeMo myBPN = null;
//        String bpnClassName = "jaNet.backprop.gui.BPNgui";
//        
//        try{
//			// try to load and create a new instance of a class which the name is specified in fnclassname string.
//			myBPN = (NNeMo) Class.forName(bpnClassName).newInstance();
//		}catch (InstantiationException instex){
//			System.out.println("Main: Error in instantiation "+bpnClassName+".class ("+instex+").");
//			System.exit(0);
//		}catch (IllegalAccessException illaccex){
//			System.out.println("Main: Error in accessing "+bpnClassName+".class ("+illaccex+").");
//			System.exit(0);
//		}catch (ClassNotFoundException classnotfound){
//			System.out.println("Main: Error, "+bpnClassName+".class not found ("+classnotfound+").");
//			System.exit(0);
//		}
//		
//		if(args.length == 1){
//			myBPN.NNeMoInit("./",args[0]);
//		}else if(args.length == 1){
//			myBPN.NNeMoInit(args[0],args[1]);
//		}else{
//			//myBPN.NNeMoInit();
//		}
//    }

}


